package com.syntevo.plugin.committemplate;

import java.io.*;
import java.util.*;

import org.tmatesoft.svn.core.*;
import org.tmatesoft.svn.core.wc.*;
import org.tmatesoft.svn.core.wc2.*;

import com.syntevo.openapi.smartsvn.command.*;
import com.syntevo.openapi.smartsvn.command.commit.*;

/**
 * @author syntevo GmbH
 */
final class CommitTemplateCommitWorkflow extends FilterCommitWorkflow {

	// Constants ==============================================================

	private static final String COMMIT_TEMPLATE_PROPERTY_NAME = System.getProperty("commitTemplate.svnPropertyName", "tsvn:logtemplate");

	// Fields =================================================================

	private final CommitWorkflowServices services;

	// Setup ==================================================================

	public CommitTemplateCommitWorkflow(ICommitWorkflow baseWorkflow, CommitWorkflowServices services) {
		super(baseWorkflow);
		this.services = services;
	}

	// Implemented ============================================================

	@Override
	public Set<CommitPacket> prepareForPacketPhases(Set<CommitPacket> packets) {
		for (final CommitPacket packet : packets) {
			if (packet.getMessage() == null) {
				packet.setMessage(getTemplateMessage(packet.getCommitRoot().getFile()));
			}
		}
		return super.prepareForPacketPhases(packets);
	}

	// Utils ==================================================================

	private String getTemplateMessage(File root) {
		final SvnClients clients = services.getClients();
		final SVNPropertyData propertyData;
		try {
			final SVNPropertyData[] data = new SVNPropertyData[1];
			final SvnOperationFactory operationsFactory = clients.createOperationsFactory();
			final SvnGetProperties getProperties = operationsFactory.createGetProperties();
			getProperties.setSingleTarget(SvnTarget.fromFile(root, SVNRevision.WORKING));
			getProperties.setRevision(SVNRevision.WORKING);
			getProperties.setDepth(SVNDepth.EMPTY);
			getProperties.setReceiver(new ISvnObjectReceiver<SVNProperties>() {
			    public void receive(SvnTarget target, SVNProperties object) throws SVNException {
			        if (object.containsName(COMMIT_TEMPLATE_PROPERTY_NAME)) {
			            data[0] = new SVNPropertyData(COMMIT_TEMPLATE_PROPERTY_NAME, object.getSVNPropertyValue(COMMIT_TEMPLATE_PROPERTY_NAME), operationsFactory.getOptions());
			        }
			    }
			});
			getProperties.run();
			propertyData = data[0];
		}
		catch (SVNException ex) {
			CommitTemplatePlugin.LOGGER.error("Failed to load commit message template by reading the property " + COMMIT_TEMPLATE_PROPERTY_NAME + " from " + root, ex);
			return null;
		}

		if (propertyData == null || propertyData.getValue() == null) {
			return null;
		}

		return SVNPropertyValue.getPropertyAsString(propertyData.getValue());
	}
}
