package com.syntevo.plugin.jira.transport;

import java.io.*;
import java.net.*;
import java.security.*;
import java.util.*;

/**
 * @author syntevo GmbH
 */
class JiraDebugLoggingConnection extends HttpURLConnection {

	// Fields =================================================================

	private final HttpURLConnection connection;
	private final PrintWriter log;

	private InputStream internalIs;
	private OutputStream internalOs;
	private InputStream internalErr;
	private InputStream is;
	private OutputStream os;
	private InputStream err;

	// Setup ==================================================================

	public JiraDebugLoggingConnection(URL url, HttpURLConnection connection, PrintWriter log) {
		super(url);

		this.connection = connection;
		this.log = log;
	}

	// Implemented ============================================================

	public void connect() throws IOException {
		connection.connect();
	}

	public void disconnect() {
		connection.disconnect();
	}


	public InputStream getErrorStream() {
		final InputStream internalErr = connection.getErrorStream();
		if (internalErr == this.internalErr) {
			return err;
		}

		this.internalErr = internalErr;
		this.err = new JiraDebugLoggingInputStream(internalErr, new JiraDebugLoggingLogStream(log, "ERROR: "));
		return err;
	}

	public boolean usingProxy() {
		return connection.usingProxy();
	}

	public boolean getAllowUserInteraction() {
		return connection.getAllowUserInteraction();
	}

	public int getConnectTimeout() {
		return connection.getConnectTimeout();
	}

	public Object getContent() throws IOException {
		return connection.getContent();
	}

	public Object getContent(Class[] classes) throws IOException {
		return connection.getContent(classes);
	}

	public String getContentEncoding() {
		return connection.getContentEncoding();
	}

	public int getContentLength() {
		return connection.getContentLength();
	}

	public String getContentType() {
		return connection.getContentType();
	}

	public long getDate() {
		return connection.getDate();
	}

	public boolean getDefaultUseCaches() {
		return connection.getDefaultUseCaches();
	}

	public boolean getDoInput() {
		return connection.getDoInput();
	}

	public boolean getDoOutput() {
		return connection.getDoOutput();
	}

	public long getExpiration() {
		return connection.getExpiration();
	}

	public String getHeaderField(int n) {
		return connection.getHeaderField(n);
	}

	public String getHeaderField(String name) {
		return connection.getHeaderField(name);
	}

	public long getHeaderFieldDate(String name, long Default) {
		return connection.getHeaderFieldDate(name, Default);
	}

	public int getHeaderFieldInt(String name, int Default) {
		return connection.getHeaderFieldInt(name, Default);
	}

	public String getHeaderFieldKey(int n) {
		return connection.getHeaderFieldKey(n);
	}

	public boolean getInstanceFollowRedirects() {
		return connection.getInstanceFollowRedirects();
	}

	public Map<String, List<String>> getHeaderFields() {
		return connection.getHeaderFields();
	}

	public long getIfModifiedSince() {
		return connection.getIfModifiedSince();
	}

	public InputStream getInputStream() throws IOException {
		final InputStream internalIs = connection.getInputStream();
		if (internalIs == this.internalIs) {
			return is;
		}

		this.internalIs = internalIs;
		this.is = new JiraDebugLoggingInputStream(internalIs, new JiraDebugLoggingLogStream(log, "READ: "));
		return is;
	}

	public long getLastModified() {
		return connection.getLastModified();
	}

	public OutputStream getOutputStream() throws IOException {
		final OutputStream internalOs = connection.getOutputStream();
		if (internalOs == this.internalOs) {
			return os;
		}

		this.internalOs = internalOs;
		this.os = new JiraDebugLoggingOutputStream(internalOs, new JiraDebugLoggingLogStream(log, "WRITE: "));
		return os;
	}

	public Permission getPermission() throws IOException {
		return connection.getPermission();
	}

	public String getRequestMethod() {
		return connection.getRequestMethod();
	}

	public int getResponseCode() throws IOException {
		return connection.getResponseCode();
	}

	public String getResponseMessage() throws IOException {
		return connection.getResponseMessage();
	}

	public void setChunkedStreamingMode(int chunklen) {
		connection.setChunkedStreamingMode(chunklen);
	}

	public void setFixedLengthStreamingMode(int contentLength) {
		connection.setFixedLengthStreamingMode(contentLength);
	}

	public void setInstanceFollowRedirects(boolean followRedirects) {
		connection.setInstanceFollowRedirects(followRedirects);
	}

	public void setRequestMethod(String method) throws ProtocolException {
		connection.setRequestMethod(method);
	}

	public int getReadTimeout() {
		return connection.getReadTimeout();
	}

	public Map<String, List<String>> getRequestProperties() {
		return connection.getRequestProperties();
	}

	public String getRequestProperty(String key) {
		return connection.getRequestProperty(key);
	}

	public URL getURL() {
		return connection.getURL();
	}

	public boolean getUseCaches() {
		return connection.getUseCaches();
	}

	public void setAllowUserInteraction(boolean allowuserinteraction) {
		connection.setAllowUserInteraction(allowuserinteraction);
	}

	public void setConnectTimeout(int timeout) {
		connection.setConnectTimeout(timeout);
	}

	public void setDefaultUseCaches(boolean defaultusecaches) {
		connection.setDefaultUseCaches(defaultusecaches);
	}

	public void setDoInput(boolean doinput) {
		connection.setDoInput(doinput);
	}

	public void setDoOutput(boolean dooutput) {
		connection.setDoOutput(dooutput);
	}

	public void setIfModifiedSince(long ifmodifiedsince) {
		connection.setIfModifiedSince(ifmodifiedsince);
	}

	public void setReadTimeout(int timeout) {
		connection.setReadTimeout(timeout);
	}

	public void setRequestProperty(String key, String value) {
		connection.setRequestProperty(key, value);
	}

	public void setUseCaches(boolean usecaches) {
		connection.setUseCaches(usecaches);
	}

	public String toString() {
		return connection.toString();
	}

	public void addRequestProperty(String key, String value) {
		connection.addRequestProperty(key, value);
	}
}
