package com.syntevo.plugin.jira.transport;

import sun.security.validator.*;

import java.security.*;
import java.security.cert.*;
import javax.net.ssl.*;

/**
 * @author syntevo GmbH
*/
final class JiraHttpsTrustManager implements X509TrustManager {

	// Fields =================================================================

	private final JiraConnection connection;

	// Setup ==================================================================

	public JiraHttpsTrustManager(JiraConnection connection) {
		this.connection = connection;
	}

	// Implemented ============================================================

	public void checkClientTrusted(X509Certificate[] certificates, String string) throws CertificateException {
	}

	public void checkServerTrusted(X509Certificate[] certificates, String string) throws CertificateException {
		if (certificates.length == 0) {
			throw new ValidatorException("Can't trust empty certificate list.");
		}

		if (!connection.checkSSLServerFingerprintTrusted(getFingerprint(certificates[0], "SHA"), getFingerprint(certificates[0], "MD5"), connection.getUrl())) {
			throw new ValidatorException("Server not trusted.");
		}
	}

	public X509Certificate[] getAcceptedIssuers() {
		return new X509Certificate[0];
	}

	// Utils ==================================================================

	private static String getFingerprint(X509Certificate certificate, String algorithm) throws CertificateException {
		final byte[] encodedBytes = certificate.getEncoded();
		final MessageDigest messagedigest;
		try {
			messagedigest = MessageDigest.getInstance(algorithm);
		}
		catch (NoSuchAlgorithmException ex) {
			throw new CertificateException("Can't create MD5-digest!");
		}

		final byte[] digestedBytes = messagedigest.digest(encodedBytes);
		final StringBuffer buffer = new StringBuffer();
		for (int index = 0; index < digestedBytes.length; index++) {
			byte2hex(digestedBytes[index], buffer);
			if (index < digestedBytes.length - 1) {
				buffer.append(":");
			}
		}

		return buffer.toString();
	}

	private static void byte2hex(byte b, StringBuffer buffer) {
		final char[] ac = {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', 'C', 'D', 'E', 'F'};
		final int i = (b & 0xf0) >> 4;
		final int j = b & 0xf;
		buffer.append(ac[i]);
		buffer.append(ac[j]);
	}
}